div {
  display: inline;
}

span {
  display: block;
}


//#  sourceMappingURL=sourcemap1.css.map

//@ sourceMappingURL=sourcemap2.css.map  

/*#  sourceMappingURL=sourcemap3.css.map */

/*@ sourceMappingURL=sourcemap4.css.map  */

//#  sourceURL=sourcemap5.css.map

//@ sourceURL=sourcemap6.css.map  

/*#  sourceURL=sourcemap7.css.map */

/*@ sourceURL=sourcemap8.css.map  */
